/**************************************************************************
*
*	File: generic_factory.h
*
*	This file is basically what Herb Sutter and Jim Hyslop show in an article
*	on http://www.cuj.com/ so I can't really take any credit for it.
*
*
*	DISCLAIMER AND STUFF:
*	This source-code is supplied AS IS. I cannot be held responsible for any damage caused
*	by use or misuse of this source-code nor any derivatives of it.
*	So use the code as you please, but give credit where they are due.
*
**************************************************************************/
#ifndef __GENERIC_FACTORY_H__
#define __GENERIC_FACTORY_H__

#include <map>
#include <string>
#include "handle.h"

template <class prod_type, typename class_id = std::string>
class generic_factory
{
	typedef handle<prod_type> (*base_type_fn)();
	typedef std::map<class_id, base_type_fn> registry_t;
	registry_t registry;
	generic_factory() {}
	generic_factory(const generic_factory &); // not implemented
	generic_factory &operator=(const generic_factory &); // not implemented
public:
	static generic_factory &instance()
	{
		static generic_factory gf;
		return gf;
	}
	void register_type(const class_id &key, base_type_fn fn)
	{
		registry[key] = fn;
	}
	handle<prod_type> create(const class_id &key) const
	{
		handle<prod_type> obj;
		registry_t::const_iterator entry = registry.find(key);
		if(entry != registry.end())
			obj = entry->second();
		return obj;
	}
};

template <class parent_t, class self_t, typename classid_t = std::string>
class register_in_factory
{
public:
	static handle<parent_t> create_instance()
	{
		return handle<parent_t>(new self_t);
	}
	register_in_factory(const classid_t &key)
	{
		generic_factory<parent_t>::instance().register_type(key, create_instance);
	}
};


#endif

