/*************************************************************************
*
*	File: image.cpp
*	Copyright(c) Andreas Magnusson 2002
*
*	This is the image implementation...
*
*	DISCLAIMER AND STUFF:
*	This source-code is supplied AS IS. I cannot be held responsible for any damage caused
*	by use or misuse of this source-code nor any derivatives of it.
*	So use the code as you please, but give credit where they are due.
*
*************************************************************************/

// disable the annoying warnings that comes up when you use map...
#pragma warning( disable : 4786)
#include "image.h"
#include <stdio.h>

image::image()
:width(0), height(0), buffer(0), color_depth(0)
{
}

image::image(const image &img)
:width(img.width), height(img.height), color_depth(img.color_depth)
{
	int size = height * width * (color_depth >> 3);
	buffer = new unsigned char[size];
	memcpy(buffer, img.buffer, size);
}

image::~image()
{
	if(buffer)
		delete[] buffer;
}

bool image::copy(const image &img)
{
	width = img.get_width();
	height = img.get_height();
	color_depth = img.get_color_depth();
	long size = height * width * (color_depth >> 3);
	buffer = new unsigned char[size];
	if(!buffer)
		return false;
	memcpy(buffer, img.get_bits(), size);
	return true;
}

///////////////////////////////

image_hnd load_image(const std::string &file_name)
{
	int p = file_name.find_last_of('.');
	std::string suffix;
	if(p != std::string::npos)
		suffix = file_name.substr(p + 1);

	image_hnd img = image_factory::instance().create(suffix);
	if(img.is_null())
		return img;

	if(img->load(file_name))
		return img;

	return handle<image>(0);
}

bool save_image(const std::string &file_name, image_hnd &src)
{
	int p = file_name.find_last_of('.');
	std::string suffix;
	if(p != std::string::npos)
		suffix = file_name.substr(p + 1);

	if(_stricmp(suffix.c_str(), src->get_type()) != 0)
	{
		image_hnd img = image_factory::instance().create(suffix);
		if(img.is_null())
			return false;

		if(!img->copy(*src))
			return false;

		return img->save(file_name);
	}
	else
		return src->save(file_name);
}

